//////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DSDummyTypeExtraction.cpp
//

#include <stdio.h>
#include "momusys.h"
#include "inter.h"
#include "intra.h"
#include "fifo.h"
#include "DescriptionSchemes.h"         /* Naming Convention */
#include "ExtractionUtilities.h"
#include "readparam.h"

//static int initextractionwascalled=0;


//=============================================================================
	

using namespace XM;


const UUID DSDummyTypeExtractionInterface::myID = UUID("");
const char *DSDummyTypeExtractionInterface::myName =
"DSDummyTypeDescriptionExtractorInterface";

const UUID DSDummyTypeExtractionTool::myID = UUID("");
const char *DSDummyTypeExtractionTool::myName =
"DSDummyTypeDescriptionExtractor";

const UUID DSDummyTypeExtractionInterfaceABC::myID = UUID();

//=============================================================================
DSDummyTypeExtractionInterfaceABC::DSDummyTypeExtractionInterfaceABC()
{
  SetInterfaceID(myID);
}

//=============================================================================
DSDummyTypeExtractionInterface::
DSDummyTypeExtractionInterface(DSDummyTypeExtractionTool* aTool):
m_ExtractionTool(aTool)
{
  SetInterfaceID(myID);
}

//----------------------------------------------------------------------------
DSDummyTypeExtractionInterface::~DSDummyTypeExtractionInterface()
{
}

//----------------------------------------------------------------------------
const UUID& DSDummyTypeExtractionInterface::GetInterfaceID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DSDummyTypeExtractionInterface::GetName(void)
{ return myName; }


//----------------------------------------------------------------------------
void DSDummyTypeExtractionInterface::destroy(void)
{ delete m_ExtractionTool; }

//----------------------------------------------------------------------------
// This informs the extractor where the source data is coming from. This
// could be an audio file, image stream etc. Here it's just a filename.
int DSDummyTypeExtractionInterface::SetSourceMedia(MultiMediaInterfaceABC* media)
{
  return m_ExtractionTool->SetSourceMedia(media);
}

//----------------------------------------------------------------------------
GenericDSInterfaceABC* DSDummyTypeExtractionInterface::
GetDescriptionInterface(void)

{
  return m_ExtractionTool->GetDescriptionInterface();
}

//----------------------------------------------------------------------------
int DSDummyTypeExtractionInterface::
SetDescriptionInterface(GenericDSInterfaceABC
		       *aDSDummyTypeDescriptionInterface)

{
  return m_ExtractionTool->
    SetDescriptionInterface(aDSDummyTypeDescriptionInterface);
}

//----------------------------------------------------------------------------
unsigned long DSDummyTypeExtractionInterface::InitExtracting(void)

{
  return(m_ExtractionTool->InitExtracting());
}

//----------------------------------------------------------------------------
unsigned long DSDummyTypeExtractionInterface::StartExtracting(void)

{
  return m_ExtractionTool->StartExtracting();
}

//----------------------------------------------------------------------------
unsigned long DSDummyTypeExtractionInterface::PostExtracting(void)

{
  return m_ExtractionTool->PostExtracting();
}

//=============================================================================
DSDummyTypeExtractionTool::DSDummyTypeExtractionTool():
m_Interface(this),
m_DescriptionInterface(NULL),
m_Media(NULL),
m_FrameCnt(0)

{
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* create sub Extractions and
     connect them with descriptor if availabe*/
  SubDSDummyTypeAExtraction
    *aSubDSDummyTypeAExtraction= new SubDSDummyTypeAExtraction();
  SubDSDummyTypeBExtraction
    *aSubDSDummyTypeBExtraction= new SubDSDummyTypeBExtraction();

  /* connect sub Extractions with DSDummyType Extraction*/
  if (aSubDSDummyTypeAExtraction)
    m_SubDSDummyTypeAExtraction=aSubDSDummyTypeAExtraction->
      GetInterface();
  if (aSubDSDummyTypeBExtraction)
    m_SubDSDummyTypeBExtraction=aSubDSDummyTypeBExtraction->
      GetInterface();
#endif /* __HasSubTypes*/
  
  m_FrameCnt=0;
  SetExposedInterface(&m_Interface);
}

//=============================================================================
DSDummyTypeExtractionTool::
DSDummyTypeExtractionTool(GenericDSInterfaceABC
			*DSDummyType):
m_Interface(this),
m_DescriptionInterface(NULL),
m_Media(NULL),
m_FrameCnt(0)

{
  // connect Descritors with Extractions*/
  SetDescriptionInterface(DSDummyType);

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/

  /* create sub Extraction and
     connect them with descriptor if availabe*/
  /* create sub Extractions and
     connect them with descriptor if availabe*/
  SubDSDummyTypeAExtraction
    *aSubDSDummyTypeAExtraction= new SubDSDummyTypeAExtraction();
  SubDSDummyTypeBExtraction
    *aSubDSDummyTypeBExtraction= new SubDSDummyTypeBExtraction();

  /* connect sub Extractions with DSDummyType Extraction*/
  if (aSubDSDummyTypeAExtraction)
    m_SubDSDummyTypeAExtraction=aSubDSDummyTypeAExtraction->
      GetInterface();
  if (aSubDSDummyTypeBExtraction)
    m_SubDSDummyTypeBExtraction=aSubDSDummyTypeBExtraction->
      GetInterface();

  /* connection to description is done in extraction*/
#endif /* __HasSubTypes*/

  SetExposedInterface(&m_Interface);
}

//----------------------------------------------------------------------------
DSDummyTypeExtractionTool::~DSDummyTypeExtractionTool()
{
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDSDummyTypeAExtraction) m_SubDSDummyTypeAExtraction->destroy();
  if (m_SubDSDummyTypeBExtraction) m_SubDSDummyTypeBExtraction->destroy();
#endif /* __HasSubTypes*/
}

//----------------------------------------------------------------------------
bool DSDummyTypeExtractionTool::SupportsPush(void)
{ return true; }
//----------------------------------------------------------------------------
bool DSDummyTypeExtractionTool::SupportsPull(void)
{ return false; }

//----------------------------------------------------------------------------
// This informs the extractor where the source data is coming from. This
// could be an audio file, image stream etc. Here it's just a filename.
int DSDummyTypeExtractionTool::SetSourceMedia(MultiMediaInterfaceABC* media)
{
  if(media==NULL) return -1;
  m_Media=media;
  return 0;
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDSDummyTypeAExtraction)
    m_SubDSDummyTypeAExtraction->SetSourceMedia(media);
  if (m_SubDSDummyTypeBExtraction)
    m_SubDSDummyTypeBExtraction->SetSourceMedia(media);
#endif /* __HasSubTypes*/
}


//----------------------------------------------------------------------------
GenericDSInterfaceABC* DSDummyTypeExtractionTool::
GetDescriptionInterface(void)

{
  return m_DescriptionInterface;
}

//----------------------------------------------------------------------------
int DSDummyTypeExtractionTool::
SetDescriptionInterface(GenericDSInterfaceABC
		       *ParentDSInterface)

{
  if (!ParentDSInterface) return -1;

  m_DescriptionInterface=ParentDSInterface;
  return 0;
}

//----------------------------------------------------------------------------
// This initialises the extraction process. Bascially it should 
// reset all storage etc. However, as this hasn't got any storage
// it's a bit pointless.
unsigned long DSDummyTypeExtractionTool::InitExtracting(void)

{
  /* test if parent exists*/
  if (!m_DescriptionInterface)  return (unsigned long)-1;

  /* test if parent is document node. If yes, remove old DSDummyType*/
  if (m_DescriptionInterface->GetDSName() == "#document") {
    if (!m_DSDummyType.isNull()) {
      if (m_DSDummyType.GetDSName() == "DSDummyType") {
	m_DescriptionInterface->RemoveChild(m_DSDummyType);
      }
    }
  }

  /* create DSDummyType*/
  m_DSDummyType=m_DescriptionInterface->
    CreateChild("DSDummyType");

  /* example for initializing extraction paramters*/
  m_DummyExtractionParameter=atoi(getpara("ParameterNameInFile","1"/*default
								     value*/));

  /* initialize description values*/
  m_FrameCnt=0;

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/

  /* this can also be done in StartExtracting if multiple instances
     of the sub-types are required*/

  /* init extraction of SubDSDummyTypeA description*/
  m_SubDSDummyTypeAExtractionInterface->
    SetDescription(m_DSDummyType->GetInterface());
  m_SubDSDummyTypeAExtractionInterface->InitExtracting();

  /* init extraction of SubDSDummyTypeB description*/
  m_SubDSDummyTypeBExtractionInterface->
    SetDescription(m_DSDummyType->GetInterface());
  m_SubDSDummyTypeBExtractionInterface->InitExtracting();
#endif /* __HasSubTypes*/

  return(0UL);
}

//----------------------------------------------------------------------------
// This uses the m_Media pointer to extract the relevant data and put it
// into the 
unsigned long DSDummyTypeExtractionTool::StartExtracting(void)

{
  // select media type you require and remove this comment
  MomVop *ImageMedia;
  //  AudioType *AudioMedia;  // type of audio media still needs to be defined

  // First check that it's all correctly initialised
  if (m_DSDummyType.isNull()) return (unsigned long)-1;
  if (m_DSDummyType.GetDSName() != "DSDummyType") return (unsigned long)-1;


  if (m_Media==NULL) return (unsigned long)-1;
  if (ImageMedia==NULL) return (unsigned long)-1;

  // Extract the value we want. For the dummy it's just the number of frames
  // in the Sequence multiplied with the extraction parameter value

  /* Increment frame counter*/
  m_FrameCnt++;
  //  const long int val=  m_FrameCnt*
  //  m_DummyExtractionParameter;
  
  /*   write extracted data to Description*/
  // m_DSDummyType->SetIntAtribute(FrameCounter,val);
  // in this example this is done in the PostExtracting function

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* uses this if multiple instances of this sub-type are needed*/
  //m_SubDSDummyTypeAExtractionInterface->
  //  SetDescription(m_DSDummyType->GetInterface());
  //m_SubDSDummyTypeAExtractionInterface->InitExtracting();
  /* extraction of SubDSDummyTypeA description*/
  m_SubDSDummyTypeAExtractionInterface->StartExtracting();

  /* uses this if multiple instances of this sub-type are needed*/
  //m_SubDSDummyTypeBExtractionInterface->
  //  SetDescription(m_DSDummyType->GetInterface());
  //m_SubDSDummyTypeBExtractionInterface->InitExtracting();
  /* extraction of SubDSDummyTypeB description*/
  m_SubDSDummyTypeBExtractionInterface->StartExtracting();
#endif /* __HasSubTypes*/

  // All done
  return 0;
}

//----------------------------------------------------------------------------
// This initialises the extraction process. Bascially it should 
// reset all storage etc. However, as this hasn't got any storage
// it's a bit pointless.
unsigned long DSDummyTypeExtractionTool::PostExtracting(void)

{
  int val=  m_FrameCnt*
    m_DummyExtractionParameter;
  
  /*   write extracted data to Description*/
  m_DSDummyType.SetAttribute("FrameCounter",val);

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* post extraction of SubDSDummyTypeA description*/
  m_SubDSDummyTypeAExtractionInterface->PostExtracting();

  /* post extraction of SubDSDummyTypeB description*/
  m_SubDSDummyTypeBExtractionInterface->PostExtracting();
#endif /* __HasSubTypes*/

  return(0UL);
}

//----------------------------------------------------------------------------
const UUID& DSDummyTypeExtractionTool::GetObjectID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DSDummyTypeExtractionTool::GetName(void)
{ return myName; }

//----------------------------------------------------------------------------
DSDummyTypeExtractionInterfaceABC *DSDummyTypeExtractionTool::GetInterface(void)
{ return &m_Interface; }

