///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DSDummyTypeSearch.h
//

#ifndef __DSDUMMYTYPESEARCH_H__
#define __DSDUMMYTYPESEARCH_H__

/* remove this, only for test purpose*/
#define __SearchAndRetrievalTypeApplication
//#define __ImageProcessingTypeApplication

#include "XMInterfaces.h"                            /* Naming Convention */
#include "XMObjects.h"                               /* Naming Convention */
#include "Descriptors.h"
#include "DescriptionSchemes.h"
#include "Media.h"

//=============================================================================

namespace XM
{


// Forward Declarations:
class DSDummyTypeSearchTool;

//=============================================================================
class DSDummyTypeSearchInterfaceABC: public I_SearchInterfaceABC
{
public:
  DSDummyTypeSearchInterfaceABC();
  virtual ~DSDummyTypeSearchInterfaceABC() {};

  virtual void destroy() = 0;

  virtual int SetRefDescriptionInterface
    (GenericDSInterfaceABC *aDSDummyTypeDescriptionInterface) = 0;
  virtual GenericDSInterfaceABC* GetQueryDescriptionInterface(void) = 0;
  virtual int SetQueryDescriptionInterface
    (GenericDSInterfaceABC *aDSDummyTypeDescriptionInterface) = 0;

  /* remove the sections you dont need. One section is needed in minimum!*/
#ifdef __SearchAndRetrievalTypeApplication
  virtual double GetDistance(void) = 0;
#endif
#ifdef __ImageProcessingTypeApplication
  virtual int SetMedia(MultiMediaInterfaceABC* media) = 0;
  virtual double InitSearch(void) = 0;
  virtual double StartSearch(void) = 0;
  virtual double PostSearch(void) = 0;
#endif
	
  /* sub-components do not have get and set functions because
     there is only one output value*/

  static const UUID myID;
};

//=============================================================================
class DSDummyTypeSearchInterface: 
public DSDummyTypeSearchInterfaceABC
{
public:
  DSDummyTypeSearchInterface(DSDummyTypeSearchTool* aTool);
  virtual ~DSDummyTypeSearchInterface();
	
  virtual const UUID& GetInterfaceID(void);
  virtual const char *GetName(void);

  virtual void destroy();

  virtual int SetRefDescriptionInterface
    (GenericDSInterfaceABC *aDSDummyTypeDescriptionInterface);
  virtual GenericDSInterfaceABC* GetQueryDescriptionInterface(void);
  virtual int SetQueryDescriptionInterface
    (GenericDSInterfaceABC *aDSDummyTypeDescriptionInterface);

  /* remove the sections you dont need. One section is needed in minimum!*/
#ifdef __SearchAndRetrievalTypeApplication
  virtual double GetDistance(void);
#endif
#ifdef __ImageProcessingTypeApplication
  virtual int SetMedia(MultiMediaInterfaceABC* media);
  virtual double InitSearch(void);
  virtual double StartSearch(void);
  virtual double PostSearch(void);
#endif

  static const UUID myID;
  static const char *myName;

private:
  DSDummyTypeSearchTool *m_SearchTool;
};

//=============================================================================
class DSDummyTypeSearchTool: public Search
{
friend DSDummyTypeSearchInterface;
public:
  DSDummyTypeSearchTool();
  DSDummyTypeSearchTool(GenericDSInterfaceABC
			*aQueryDescriptionInterface);

  virtual const UUID& GetObjectID(void);
  virtual const char *GetName(void);

  virtual DSDummyTypeSearchInterfaceABC *GetInterface(void);

  virtual bool SupportsPush(void);
  virtual bool SupportsPull(void);

  virtual int SetRefDescriptionInterface
    (GenericDSInterfaceABC *aDSDummyTypeDescriptionInterface);
  virtual GenericDSInterfaceABC* GetQueryDescriptionInterface(void);
  virtual int SetQueryDescriptionInterface
    (GenericDSInterfaceABC *aDSDummyTypeDescriptionInterface);

  /* remove the sections you dont need. One section is needed in minimum!*/
#ifdef __SearchAndRetrievalTypeApplication
  virtual double GetDistance(void);
#endif
#ifdef __ImageProcessingTypeApplication
  virtual int SetMedia(MultiMediaInterfaceABC* media); /* needed ,e.g.,
	     to read the time of the image*/
  virtual double InitSearch(void);
  virtual double StartSearch(void);
  virtual double PostSearch(void);
#endif

  static const UUID myID;
  static const char * myName;
private:
  virtual ~DSDummyTypeSearchTool();

  DSDummyTypeSearchInterface m_Interface;
  GenericDSInterfaceABC *m_RefDescriptionInterface;
  GenericDSInterfaceABC *m_QueryDescriptionInterface;
#ifdef __ImageProcessingTypeApplication
 MultiMediaInterfaceABC* m_Media;
#endif
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  SubDSDummyTypeBSearchInterfaceABC *m_SubDSDummyTypeBSearch;
  SubDSDummyTypeBSearchInterfaceABC *m_SubDSDummyTypeBSearch;
#endif
};

};

#endif //__DSDUMMYTYPESEARCH_H__
