///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Stephan Herrmann, Munich University of Technology
// (contributing organizations names)
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties
// from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// XM_Main.cpp


#include <stdio.h>
#include <string.h>

#include "ComponentDB.h"
#include "readparam.h"
#include "ServerApplications.h"
#include "ClientApplications.h"
/*#include "Descriptors.h"
#include "ExtractionUtilities.h"
#include "CodingSchemes.h"*/
#include "EvaluationTools.h"
#include "Utilities.h"
#include "Version1.h"

//#define BITSTREAMTEST
#ifdef BITSTREAMTEST
#include "Encoders.h"
#include "Decoders.h"
#endif


//---------------------------------------------------------------------------
TXMApplicationParameterSet *parse_cmdline (int argc, char **argv)
{
 //   XM::ComponentDatabase  theComponentDatabase;
  static TXMApplicationParameterSet ParameterSet;
 //  char *ListFile=0;
 //char *Query=0;
 //char *ParameterFileName=0;
 //char *Bitstream=0;
 //char *Application=0;
 //int  NoOfMatches=0;
 //char *LogRedirection=0;
  char *a = 0;

#ifdef XYUVDISP
  XYUV_INIT (argc, argv);
#endif
 /* check parameters of program call */
 /* check if help will be printed */
  for (int i = 0; i < argc; i++) {
    if (!strcmp (argv[i], "-h")) {
      return 0;
    }
    if (!strcmp (argv[i], "h")) {
      return 0;
    }
    if (!strcmp (argv[i], "-?")) {
      return 0;
    }
    if (!strcmp (argv[i], "?")) {
      return 0;
    }

   /* check stderr redirection parameter */
    if (strstr (argv[i], "-r") == argv[i]) {
      ParameterSet.LogRedirection = &(argv[i][2]);
      if (*ParameterSet.LogRedirection)
	continue;
     /* redirection name in next parameter */
      ParameterSet.LogRedirection = 0;
      if ((i + 1) == argc) {
	return 0;
      }
      if (argv[i + 1][0] == '-') {
	return 0;
      }
      i++;
      ParameterSet.LogRedirection = &(argv[i][0]);
      freopen (ParameterSet.LogRedirection, "a+", stderr);
    }

   /* check application parameter */
    if (strstr (argv[i], "-a") == argv[i]) {
      ParameterSet.Application = &(argv[i][2]);
      if (*ParameterSet.Application)
	continue;
     /* application name in next parameter */
      ParameterSet.Application = 0;
      if ((i + 1) == argc) {
	return 0;
      }
      if (argv[i + 1][0] == '-') {
	return 0;
      }
      i++;
      ParameterSet.Application = &(argv[i][0]);
    }

   /* check listfile parameter */
    if (strstr (argv[i], "-l") == argv[i]) {
      ParameterSet.ListFile = &(argv[i][2]);
      if (*ParameterSet.ListFile)
	continue;
     /* listfile name in next parameter */
      ParameterSet.ListFile = 0;
      if ((i + 1) == argc) {
	return 0;
      }
      if (argv[i + 1][0] == '-') {
	return 0;
      }
      i++;
      ParameterSet.ListFile = &(argv[i][0]);
    }

   /* check bitstream parameter */
    if (strstr (argv[i], "-b") == argv[i]) {
      ParameterSet.Bitstream = &(argv[i][2]);
      if (*ParameterSet.Bitstream)
	continue;
     /* bitstream name in next parameter */
      ParameterSet.Bitstream = 0;
      if ((i + 1) == argc) {
	return 0;
      }
      if (argv[i + 1][0] == '-') {
	return 0;
      }
      i++;
      ParameterSet.Bitstream = &(argv[i][0]);
    }

   /* check parammeterfile parameter */
    if (strstr (argv[i], "-p") == argv[i]) {

      a = &(argv[i][2]);
      if (!(*a)) {

       /* query name in next parameter */
	if ((i + 1) == argc) {
	  return 0;
	}
	if (argv[i + 1][0] == '-') {
	  return 0;
	}
	i++;
	a = &(argv[i][0]);
      };
      if (*a) {
       /* read parameter file */
	readparameters (a);
      }
      if (!ParameterSet.ParameterFileName)
	ParameterSet.ParameterFileName = a;
    }

   /* check query parameter */
    if (strstr (argv[i], "-q") == argv[i]) {
      ParameterSet.Query = &(argv[i][2]);
      if (*ParameterSet.Query)
	continue;
     /* query name in next parameter */
      ParameterSet.Query = 0;
      if ((i + 1) == argc) {
	return 0;
      }
      if (argv[i + 1][0] == '-') {
	return 0;
      }
      i++;
      ParameterSet.Query = &(argv[i][0]);
    }

   /* check noofmatches parameter */
    if (strstr (argv[i], "-n") == argv[i]) {
      a = &(argv[i][2]);
      if (!*a) {
       /* noofmatches name in next parameter */
	a = 0;
	if ((i + 1) == argc) {
	  return 0;
	}
	if (argv[i + 1][0] == '-') {
	  return 0;
	}
	i++;
	a = &(argv[i][0]);
      }
      ParameterSet.NoOfMatches = atoi (a);
      if (ParameterSet.NoOfMatches <= 0)
	ParameterSet.NoOfMatches = 1;
    }
  }

  fprintf (stderr, "MPEG-7 XM Version ");
  fprintf (stderr, VERSION "\n");

 /* if paramters are not set, get them from parameter file */
  if (ParameterSet.ParameterFileName) {
    if (!ParameterSet.Application)
      ParameterSet.Application = getpara ("Application", (char *) NULL);
    if (!ParameterSet.ListFile)
      ParameterSet.ListFile = getpara ("ListFile", (char *) NULL);
    if (!ParameterSet.Bitstream)
      ParameterSet.Bitstream = getpara ("Bitstream", (char *) NULL);
    if (!ParameterSet.NoOfMatches)
      ParameterSet.NoOfMatches = atoi (getpara ("NoOfMatches", "10"));
  }

  return &ParameterSet;
}


//---------------------------------------------------------------------------
int select_application (TXMApplicationParameterSet * ParameterSet)
{
  if (!ParameterSet)
    return 1;

  if (!ParameterSet->Application)
    return 1;

 /* init components */
//  theComponentDatabase.InitDB();
//  theComponentDatabase.CreateDDL();
//  theComponentDatabase.CreateClient();
//  theComponentDatabase.CreateServer();
//  theComponentDatabase.CreateCS();
//  theComponentDatabase.CreateDecoder();
//  theComponentDatabase.CreateDS();
//  theComponentDatabase.CreateD();
//  theComponentDatabase.CreateEncoder();
//  theComponentDatabase.CreateET();
//  theComponentDatabase.CreateExtractor();
//  theComponentDatabase.CreateParser();
//  theComponentDatabase.CreateSearch();
//  theComponentDatabase.CreateSystem();

//  theComponentDatabase.GetClientApps();
//  theComponentDatabase.GetServerApps();

 /* parse application database and select application */

 /* print list of possible applications */
  if (!strcmp (ParameterSet->Application, "PrintApplications")) {
#ifdef USEDTDS
    printf ("DSDummyTypeServer\n");
    printf ("DSDummyTypeClient\n");
#endif
    return 0;
  }


#ifdef USEDTDS
 /* DSDummyType database extraction */
  if (!strcmp (ParameterSet->Application, "DSDummyTypeServer")) {
    if (!(ParameterSet->ListFile && ParameterSet->Bitstream))
      return 1;
    else {
      XM::DSDummyTypeServer theServer;

      theServer.Open ();
      theServer.Start (ParameterSet->ListFile, ParameterSet->Bitstream);
      theServer.Stop ();
      theServer.Close ();

      return 0;
    }
  }

 /* DSDummyType query retrieval */
  if (!strcmp (ParameterSet->Application, "DSDummyTypeClient")) {
    XM::DSDummyTypeClient theClient;

    if (!(ParameterSet->ListFile && ParameterSet->Bitstream && ParameterSet->Query
	&& ParameterSet->NoOfMatches))
      return 1;
    else {
      theClient.Open (ParameterSet->ListFile, ParameterSet->Bitstream);
      theClient.Start (ParameterSet->Query, ParameterSet->NoOfMatches);
      theClient.Stop ();
      theClient.Close ();

      return 0;
    }
  }
#endif

  return -1;
}

//---------------------------------------------------------------------------
int printhelp (TXMApplicationParameterSet * ParameterSet, int type_of_messages)
{
  if (!ParameterSet)
    return 1;
  
  if (type_of_messages == 0) return 0;
  if (type_of_messages > 0) {

    char empty[1] = "";

    if (!ParameterSet->ParameterFileName)
      ParameterSet->ParameterFileName = empty;
    if (!ParameterSet->Application)
      ParameterSet->Application = empty;
    if (!ParameterSet->ListFile)
      ParameterSet->ListFile = empty;
    if (!ParameterSet->Bitstream)
      ParameterSet->Bitstream = empty;
    if (!ParameterSet->Query)
      ParameterSet->Query = empty;

    fprintf (stderr, "Error: Wrong parameter list\n"
	     "call XM with:\n"
	     "XMMain.exe -p Parameterfile -aApplication -lListfile -bBitstream"
	     "-qQuery -nNoOfMatches -rLogRedirection\n\n"
	     "Parameterfile = parameter file for application(%s)\n"
	     "Application   = selected XM-application(%s)\n"
	     "Listfile      = file with list of filenames of database (%s)\n"
	     "Bitstream     = MPEG-7 bitstream (%s)\n"
	     "Query         = filename of query (%s)\n"
	     "NoOfMatches   = size of list with best matches (%d)\n"
	     "LogRedirection= file where all the log messeges are going to "
	     "(%s)\n",
      ParameterSet->ParameterFileName, ParameterSet->Application,
      ParameterSet->ListFile, ParameterSet->Bitstream, ParameterSet->Query,
      ParameterSet->NoOfMatches, ParameterSet->LogRedirection);
  }
  else {
    fprintf (stderr, "Error : Application \"%s\" unknown\n",
      ParameterSet->Application);
  }

  return 0;
}
